Package: MetaNet
Type: Package
Title: Network Analysis for Omics Data
Version: 0.3.2
Authors@R: 
    person("Chen", "Peng", email ="bfzede@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-9449-7606"))
Description: Comprehensive network analysis package.
    Calculate correlation network fastly, accelerate lots of analysis by parallel computing.
    Support for multi-omics data, search sub-nets fluently.
    Handle bigger data, more than 10,000 nodes in each omics.
    Offer various layout method for multi-omics network and some interfaces to other software ('Gephi', 'Cytoscape', 'ggplot2'), easy to visualize.
    Provide comprehensive topology indexes calculation, including ecological network stability.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0), igraph (>= 1.3.5)
LazyData: true
Imports: graphics, dplyr, ggplot2 (>= 3.2.0), ggnewscale, ggrepel,
        grDevices, magrittr, reshape2, stats, tibble, utils, pcutils
        (>= 0.2.7), rlang
Suggests: pheatmap, vegan, stringr, foreach, doSNOW, snow, knitr,
        rmarkdown, prettydoc, Hmisc, gifski, ggraph, networkD3,
        ggpmisc, ggtree, treeio, circlize, jsonify, ggpubr, corrplot,
        philentropy, spatstat.random, spatstat.geom, sf
VignetteBuilder: knitr
BugReports: https://github.com/Asa12138/MetaNet/issues
URL: https://github.com/Asa12138/MetaNet
ByteCompile: true
biocViews: DataImport, Network analysis, Omics, Software, Visualization
NeedsCompilation: no
Packaged: 2026-05-26 02:59:40 UTC; asa
Author: Chen Peng [aut, cre] (ORCID: <https://orcid.org/0000-0002-9449-7606>)
Maintainer: Chen Peng <bfzede@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-26 03:10:03 UTC
Built: R 4.5.2; ; 2026-05-26 04:37:24 UTC; unix
