# Generated by roxygen2: do not edit by hand

S3method(plot,cohesion)
S3method(plot,ggig)
S3method(plot,metanet)
S3method(plot,metanet_compare)
S3method(plot,rmt_res)
S3method(plot,robust)
S3method(plot,robustness)
S3method(plot,vulnerability)
S3method(print,cohesion)
S3method(print,coors)
S3method(print,corr)
S3method(print,ggig)
S3method(print,metanet)
S3method(print,metanet_compare)
S3method(print,robust)
S3method(print,robustness)
S3method(print,vulnerability)
S3method(summary,corr)
export("%<>%")
export("%>%")
export(Cohesion)
export(RMT_threshold)
export(adjacency_similarity)
export(anno_edge)
export(anno_vertex)
export(as.ggig)
export(as.metanet)
export(as_arc)
export(as_circle_tree)
export(as_coors)
export(as_line)
export(as_multi_layer)
export(as_poly_sector)
export(as_polyarc)
export(as_polycircle)
export(as_polygon)
export(c_net_annotate)
export(c_net_build)
export(c_net_cal)
export(c_net_calculate)
export(c_net_compare)
export(c_net_difference)
export(c_net_ego)
export(c_net_filter)
export(c_net_from_edgelist)
export(c_net_highlight)
export(c_net_index)
export(c_net_intersect)
export(c_net_lay)
export(c_net_layout)
export(c_net_load)
export(c_net_module)
export(c_net_neighbors)
export(c_net_plot)
export(c_net_save)
export(c_net_set)
export(c_net_skeleton)
export(c_net_stability)
export(c_net_union)
export(c_net_update)
export(cal_sim)
export(check_tabs)
export(clean_igraph)
export(clean_multi_edge_metanet)
export(combine_n_module)
export(compare_rand)
export(df2net_tree)
export(extract_sample_net)
export(fast_cor)
export(filter_n_module)
export(fit_power)
export(g_layout)
export(g_layout_backbone)
export(g_layout_circlepack)
export(g_layout_multi_layer)
export(g_layout_nice)
export(g_layout_poly_sector)
export(g_layout_polyarc)
export(g_layout_polycircle)
export(g_layout_polygon)
export(g_layout_stress)
export(g_layout_treemap)
export(get_big_lay_nice)
export(get_community)
export(get_e)
export(get_group_skeleton)
export(get_module)
export(get_module_eigen)
export(get_n)
export(get_v)
export(input_cytoscape)
export(input_gephi)
export(is_metanet)
export(links_stat)
export(module_detect)
export(module_eigen)
export(module_expression)
export(module_net)
export(multi_net_build)
export(nc)
export(netD3plot)
export(net_par)
export(olympic_rings_net)
export(p.adjust.table)
export(plot_corr_heatmap)
export(plot_e_type_bar)
export(plot_module_tree)
export(plot_multi_nets)
export(plot_net_degree)
export(rand_net)
export(rand_net_par)
export(read_corr)
export(rmt)
export(robust_test)
export(robustness)
export(save_corr)
export(show_MetaNet_logo)
export(skeleton_plot)
export(smallworldness)
export(spatstat_layout)
export(summ_2col)
export(summary_module)
export(to_module_net)
export(transform_coors)
export(twocol_edgelist)
export(venn_net)
export(vulnerability)
export(zp_analyse)
export(zp_plot)
import(ggplot2)
import(igraph)
importFrom(dplyr,arrange)
importFrom(dplyr,count)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,symbols)
importFrom(graphics,text)
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(pcutils,change_fac_lev)
importFrom(pcutils,get_cols)
importFrom(pcutils,guolv)
importFrom(pcutils,hebing)
importFrom(pcutils,lib_ps)
importFrom(pcutils,list_to_dataframe)
importFrom(pcutils,mmscale)
importFrom(pcutils,tidai)
importFrom(pcutils,trans)
importFrom(pcutils,update_param)
importFrom(reshape2,acast)
importFrom(reshape2,melt)
importFrom(stats,aggregate)
importFrom(stats,approx)
importFrom(stats,coef)
importFrom(stats,coefficients)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,kmeans)
importFrom(stats,ks.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pt)
importFrom(stats,relevel)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,smooth.spline)
importFrom(stats,time)
importFrom(stats,var)
importFrom(tibble,column_to_rownames)
importFrom(tibble,rownames_to_column)
importFrom(utils,combn)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,packageName)
importFrom(utils,read.csv)
importFrom(utils,tail)
importFrom(utils,write.csv)
