MetaNet - free portable library for meta neural network research

INSTALLATION
------------
Make sure you have numpy installed, then
	python3 setup.py install

To remove package
	pip uninstall metanet
