---
title: "MetaNet: Network Analysis for Omics Data"
output: 
  prettydoc::html_pretty:
    toc: true
    toc_depth: 3
    theme: tactile
author: "Chen Peng"
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{MetaNet}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Install

```{r eval=FALSE}
if (!requireNamespace("devtools")) install.packages("devtools")
devtools::install_github("Asa12138/pcutils")
devtools::install_github("Asa12138/MetaNet")
```

## Vignette

The HTML documentation of the latest version is available at [Github page](https://asa12138.github.io/MetaNet/).

Please go to <https://asa12138.github.io/MetaNet_tutorial/> for the full vignette.

## Citation

Please cite:

Chen Peng (2023). MetaNet: Network Analysis for Multi-Omics Data. _R package_, <https://github.com/Asa12138/MetaNet>.

## Need helps?

If you have questions/issues, please visit
[MetaNet homepage](https://github.com/Asa12138/MetaNet) first.
Your problems are mostly documented. 

If you think you found a bug, please post on
[github issue](https://github.com/Asa12138/MetaNet/issues).
