% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-3.plot.R
\name{plot.metanet}
\alias{plot.metanet}
\title{Plot a metanet}
\usage{
\method{plot}{metanet}(x, ...)
}
\arguments{
\item{x}{metanet object}

\item{...}{add}
}
\value{
plot
}
\description{
Plot a metanet
}
