% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MetaNet-package.R
\name{show_MetaNet_logo}
\alias{show_MetaNet_logo}
\title{Show MetaNet logo}
\usage{
show_MetaNet_logo()
}
\value{
picture
}
\description{
Show MetaNet logo
}
